/*============================================================================
  File:     VerifyFragmentation.sql

  Summary:  Using sys.dm_db_index_physical_stats to view row sizes, table 
            sizes and fragmentation.
  
  Date:     November 2010

  SQL Server Version: SQL Server 2008 SP1
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  written/presented by SQLskills.com  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SELECT count(*) FROM SalesDBInts.dbo.Sales
SELECT count(*) FROM SalesDBGUIDs.dbo.Sales
SELECT count(*) FROM SalesDBSeqGUIDs.dbo.Sales
go

USE SalesDBInts
go

SELECT object_name(ps.object_id) AS TableName
	   , si.name AS IndexName
	   , ps.index_level AS IndexLevel
	   , ps.record_count AS RecordCount
	   , ps.page_count AS Pages
	   , ps.avg_fragmentation_in_percent
	   , ps.avg_page_space_used_in_percent
FROM sys.dm_db_index_physical_stats
	(db_id('SalesDBInts')
	, object_id('SalesDBInts.dbo.Sales')
	, null
	, null
	, 'detailed') AS ps
	JOIN SalesDBInts.sys.indexes AS si
		ON ps.object_id = si.object_id AND ps.index_id = si.index_id 
WHERE ps.index_level = 0
ORDER BY TableName, ps.index_id, ps.index_level
go

USE SalesDBGUIDs
go

SELECT object_name(ps.object_id) AS TableName
	   , si.name AS IndexName
	   , ps.index_level AS IndexLevel
	   , ps.record_count AS RecordCount
	   , ps.page_count AS Pages
	   , ps.avg_fragmentation_in_percent
	   , ps.avg_page_space_used_in_percent
FROM sys.dm_db_index_physical_stats
	(db_id('SalesDBGUIDs')
	, object_id('SalesDBGUIDs.dbo.Sales')
	, null
	, null
	, 'detailed') AS ps
	JOIN SalesDBGUIDs.sys.indexes AS si
		ON ps.object_id = si.object_id AND ps.index_id = si.index_id 
WHERE ps.index_level = 0
ORDER BY TableName, ps.index_id, ps.index_level
go

USE SalesDBSeqGUIDs
go
SELECT object_name(ps.object_id) AS TableName
	   , si.name AS IndexName
	   , ps.index_level AS IndexLevel
	   , ps.record_count AS RecordCount
	   , ps.page_count AS Pages
	   , ps.avg_fragmentation_in_percent
	   , ps.avg_page_space_used_in_percent
FROM sys.dm_db_index_physical_stats
	(db_id('SalesDBSeqGUIDs')
	, object_id('SalesDBSeqGUIDs.dbo.Sales')
	, null
	, null
	, 'detailed') AS ps
	JOIN SalesDBSeqGUIDs.sys.indexes AS si
		ON ps.object_id = si.object_id AND ps.index_id = si.index_id 
WHERE ps.index_level = 0
ORDER BY TableName, ps.index_id, ps.index_level
go


USE SalesDBInts
go

sp_spaceused
go

USE SalesDBGuids
go

sp_spaceused
go

USE SalesDBSeqGUIDs
go

sp_spaceused
go